
#ifndef _SERVICE_H_
#define _SERVICE_H_

#include <QtCore/QMap>
#include <QtCore/QFile>
#include <QtCore/QProcess>
#include <QtCore/QDebug>

#include <QtDBus/QDBusAbstractAdaptor>
#include <QtDBus/QDBusConnection>

class StreamWatcher {
public:
	StreamWatcher(int max = 100)
		: m_dev(0),
		  m_last(0),
		  m_max(max)
	{}
	void setDevice(QIODevice *device)
	{
		m_dev = device;
	}
	virtual void update()
	{
		if (!m_dev)
			return;
		while (!m_dev->atEnd()) {
			append(m_dev->readLine());
		}
		while (m_list.count() > m_max)
			m_list.removeFirst();
	}
	int last()
	{
		return m_last;
	}
	QStringList get(int from)
	{
		int index = m_list.count() - (m_last - from);
		if (index < 0)
			index = 0;
		return m_list.mid(index);
	}
protected:
	virtual void append(const QString &line)
	{
		m_list.append(line);
		++m_last;
	}
protected:
	QIODevice *m_dev;
	QStringList m_list;
	int m_last;
	int m_max;
};

class DBusStreamWatcher : public StreamWatcher
{
public:
	DBusStreamWatcher(int max = 100)
		: StreamWatcher(max)
	{}
	virtual void update()
	{
		StreamWatcher::update();
		filter();
	}
protected:
	void filter()
	{
		if (!m_list.empty() && (
		    m_list.last().contains("interface=com.pengutronix.jdb.SystemInfo") ||
		    m_list.last().contains("sender=" + QDBusConnection::sessionBus().baseService()))) {
			m_list.removeLast();
			--m_last;
		}
	}
	virtual void append(const QString &line)
	{
		if (!m_list.empty() && ((line[0] == ' ') || (line[0] == '"'))) {
			m_list.last().append(line);
		}
		else {
			filter();
			m_list.append(line);
			++m_last;
		}
	}
};

class SystemInfo : public QDBusAbstractAdaptor
{
	Q_OBJECT
	Q_CLASSINFO("D-Bus Interface", "com.pengutronix.jdb.SystemInfo")
public:
	SystemInfo(QObject *parent);

public slots:
	uint Date();
	void Noop();
	QString Ping(QString text);
	int CpuLoad();
	QMap<QString, QVariant> Syslog(int last);
	QMap<QString, QVariant> DBusMonitor(int last);

protected:
	virtual void timerEvent(QTimerEvent *);

private:
	int m_cpuLoad;
	int m_oldStat;
	StreamWatcher m_syslog;
	DBusStreamWatcher m_dbusMonitor;
	QFile m_syslogFile;
	QProcess m_dbusMonitorProcess;
};

#endif // _SERVICE_H_

