/* ************************************************************************

   Copyright: 2010 by Michael Olbrich <m.olbrich@pengutronix.de>

   License:
     To the extent possible under law, Pengutronix e.K. has
     waived all copyright and related or neighboring rights to
     JSON-DBus-Bridge Demo Code.

************************************************************************ */

/* ************************************************************************

#asset(hello_world/*)

************************************************************************ */

/**
 * This is the main application class of your custom application "hello_world"
 */
qx.Class.define("hello_world.Application",
{
  extend : qx.application.Standalone,



  /*
  *****************************************************************************
     MEMBERS
  *****************************************************************************
  */

  members :
  {
    /**
     * This method contains the initial application code and gets called 
     * during startup of the application
     * 
     * @lint ignoreDeprecated(alert)
     */
    main : function()
    {
      // Call super class
      this.base(arguments);

      // Enable logging in debug variant
      if (qx.core.Variant.isSet("qx.debug", "on"))
      {
        // support native logging capabilities, e.g. Firebug for Firefox
        qx.log.appender.Native;
        // support additional cross-browser console. Press F7 to toggle visibility
        qx.log.appender.Console;
      }

      /*
      -------------------------------------------------------------------------
        Below is your actual application code...
      -------------------------------------------------------------------------
      */

      // Document is the application root
      var doc = this.getRoot();
      doc.setBackgroundColor("#ffffff");

      var back = new qx.ui.basic.Image("hello_world/back.png");
      back.setScale(true);
      doc.add(back, {left:0, top: 0});


      var core = new qx.ui.container.Composite(new qx.ui.layout.Grid());
      doc.add(core, {left: 120, top: 100});
      core.getLayout().setSpacing(10);
      core.getLayout().setColumnAlign(0, "left", "middle");

      core.add( new qx.ui.basic.Label("RPC Hello World:"), {row: 0, column: 0, colSpan: 3});
      var helloLabel = new qx.ui.basic.Label("<waiting for hello>");
      core.add(helloLabel, {row: 1, column: 0, colSpan: 3});

      core.add(new qx.ui.core.Spacer(0, 55), {row: 2, column: 0});

      core.add( new qx.ui.basic.Label("RPC Echo:"), {row: 3, column: 0, colSpan: 3});
      core.add( new qx.ui.basic.Label("Send:"), {row: 4, column: 0});
      var echoEdit = new qx.ui.form.TextField();
      echoEdit.setWidth(140);
      core.add(echoEdit, {row: 4, column: 1});
      echoEdit.focus();

      var echoButton = new qx.ui.form.Button("Go");
      core.add(echoButton, {row: 4, column: 2});

      core.add( new qx.ui.basic.Label("Receive:"), {row: 5, column: 0});
      var echoLabel = new qx.ui.basic.Label("<waiting for echo>");
      core.add(echoLabel, {row: 5, column: 1, colSpan: 2});

      var rpc = new qx.io.remote.Rpc("http://localhost:8080/rpc", "com.pengutronix.jdb.Hello|/");

      // synchronous RPC call. callSync() throws an exception on error
      try {
        var val = rpc.callSync("com.pengutronix.jdb.Hello.HelloWorld");
        helloLabel.setValue(val);
      }
      catch (error) {
        helloLabel.setValue("Error: " + error);
      }

      // Add an event listener
      echoButton.addListener("execute", function(e) {
        var text = echoEdit.getValue();
        if (text == null)
          text = "";
        var handler = function(result, error) {
           if (error == null) {
             echoLabel.setValue(result);
            }
            else {
              echoLabel.setValue("Error: " + error);
            }
        }
        // asynchronous RPC call. The handler will be called with result or error == null
        rpc.callAsync(handler, "com.pengutronix.jdb.Hello.Echo", "s", text);
      });
    }
  }
});
